@@ -28,7 +28,6 @@ from elastic_blast.gcp_traits import get_machine_properties
 
 if __name__ == "__main__":
     machineType = sys.stdin.read()
-    ncpu, nram = get_machine_properties(machineType)
-    print(nram)
+    props = get_machine_properties(machineType)
+    print(props.memory)
     sys.exit(0)
-
