@@ -1,7 +1,11 @@
 #!/bin/bash -xe
 # Script to "impresonate" TC and delete GKE clusters that are lingering
 export ELB_GCP_PROJECT=ncbi-sandbox-blast
-[ $# -lt 2 ] && { echo "Usage: $0 $ELB_RESULTS $ELB_CLUSTER_NAME"; exit 1 ; }
+[ $# -lt 2 ] && { 
+    echo "Usage: $0 <ELB_RESULTS> <ELB_CLUSTER_NAME> [ELB_GCP_ZONE]"; 
+    echo "\tDefault ELB_GCP_ZONE=us-east4-b";
+    exit 1 ; 
+}
 
 export ELB_RESULTS=${1}
 export ELB_CLUSTER_NAME=${2}
