@@ -29,16 +29,22 @@ from enum import Enum
 from dataclasses import dataclass
 from bisect import bisect_left
 import math
+from typing import Optional
 from .filehelper import open_for_read
 from .constants import BLASTDB_ERROR, INPUT_ERROR, ELB_BLASTDB_MEMORY_MARGIN
-from .constants import UNKNOWN_ERROR, MolType
+from .constants import UNKNOWN_ERROR, MolType, CSP
 from .constants import ELB_S3_PREFIX, ELB_GCS_PREFIX
-from .constants import SYSTEM_MEMORY_RESERVE
+from .constants import SYSTEM_MEMORY_RESERVE, MEMORY_FOR_BLAST_HITS
+from .constants import ELB_DFLT_AWS_REGION
 from .base import DBSource, PositiveInteger, MemoryStr
-from .util import UserReportError, get_query_batch_size
+from .util import UserReportError, get_query_batch_size, ElbSupportedPrograms
 from .aws_traits import get_instance_type_offerings, get_suitable_instance_types
 from .aws_traits import get_machine_properties as aws_get_machine_properties
+from .aws_traits import create_aws_config
 from .gcp_traits import GCP_MACHINES
+from .gcp_traits import get_machine_properties as gcp_get_machine_properties
+from .db_metadata import DbMetadata
+
 
 @dataclass
 class SeqData:
@@ -52,6 +58,14 @@ class DbData(SeqData):
     """Database information"""
     bytes_to_cache_gb: float
 
+    @classmethod
+    def from_metadata(cls, db_metadata: DbMetadata):
+        """Create an object from a database metadata object"""
+        obj = cls(length = db_metadata.number_of_letters,
+                  moltype = MolType[db_metadata.dbtype.upper()],
+                  bytes_to_cache_gb = db_metadata.bytes_to_cache / (1024 ** 3))
+        return obj
+
 
 class MTMode(Enum):
     """Values for BLAST MT mode option
@@ -66,63 +80,20 @@ class MTMode(Enum):
 
 
 # MT mode 1 query length per thread
-# From https://preview.ncbi.nlm.nih.gov/books/prevqa/NBK322759/ , in particular
-# As noted above, the threading by query option works well with relatively small databases if you have a lot of queries to process.
-# For BLASTP, your input FASTA should contain at least 10,000 residues per thread, or 320,000 residues for 32 threads. 
-# For BLASTX, the FASTA should contain 30,000 bases per thread. For both BLASTP and BLASTX, we find threading by query works best 
-# if the database has fewer than 2 billion residues (e.g, swissprot). For BLASTN, your FASTA file should contain at least 2.5 
-# million bases per thread and the database should contain fewer than 14 billion residues. Threading by query also works well if
-# you limit the search, such as by -taxid or -gilist, regardless of the size of the database.
 RESIDUES_PER_THREAD = 10000
 BASES_PER_THREAD = 2500000
-BASES_PER_THREAD_BLASTX = 30000
 
 # Maximum database length for MT mode 1
 MAX_MT_ONE_DB_LENGTH_PROT = 2000000000
 MAX_MT_ONE_DB_LENGTH_NUCL = 14000000000
 
 # Number of threads for MT mode 0
-NUM_THREADS_MT_ZERO = 16
+NUM_THREADS_MT_ZERO_AWS = 16
+NUM_THREADS_MT_ZERO_GCP = 15
 
-
-def get_db_data(db: str, dbtype: MolType, source: DBSource) -> DbData:
-    """
-    Read database metadata.
-
-    Arguments:
-        db: Database name or URI
-        dbtype: Molecule type for BLASTDB
-        source: Source for NCBI provided database, ignored for a user database
-    """
-    DB_BUCKET_AWS = os.path.join(ELB_S3_PREFIX, 'ncbi-blast-databases')
-    DB_BUCKET_GCP = os.path.join(ELB_GCS_PREFIX, 'blast-db')
-    DB_BUCKET_NCBI = 'ftp://ftp.ncbi.nlm.nih.gov/blast/db'
-
-    # for user databases
-    metadata_file = f"{db}-{dbtype}-metadata.json"
-
-    # if an NCBI-provided database
-    if not db.startswith(ELB_S3_PREFIX) and not db.startswith(ELB_GCS_PREFIX):
-        if source == DBSource.AWS or source == DBSource.GCP:
-            bucket = DB_BUCKET_AWS if source == DBSource.AWS else DB_BUCKET_GCP
-            with open_for_read(f'{bucket}/latest-dir') as f:
-                metadata_file = os.path.join(f'{bucket}/{f.readline().rstrip()}', metadata_file)
-        else:
-            metadata_file = os.path.join(f'{DB_BUCKET_NCBI}', metadata_file)
-    logging.debug(f'BLASTDB metadata file: {metadata_file}')
-    try:
-        with open_for_read(metadata_file) as f:
-            db_metadata = json.load(f)
-    except:
-        msg = f'Metadata file for database: "{db}" does not exist in {source}'
-        if db.startswith(ELB_S3_PREFIX) or db.startswith(ELB_GCS_PREFIX):
-            msg += ' or you lack credentials to access this file'
-        msg += '.'
-        raise UserReportError(returncode=BLASTDB_ERROR, message=msg)
-
-    return DbData(length = int(db_metadata['number-of-letters']),
-                  moltype = MolType[db_metadata['dbtype'].upper()],
-                  bytes_to_cache_gb = int(db_metadata['bytes-to-cache']) / (1024 ** 3))
+# Maximum number of threads to use per BLAST job
+MAX_NUM_THREADS_AWS = 16
+MAX_NUM_THREADS_GCP = 15
 
 
 def get_mt_mode(program: str, options: str, db: DbData, query: SeqData) -> MTMode:
@@ -135,8 +106,6 @@ def get_mt_mode(program: str, options: str, db: DbData, query: SeqData) -> MTMod
         db: Database information
         query: Queries information
     """
-    if program == 'blastx' and query.length <= BASES_PER_THREAD_BLASTX:
-        return MTMode.ZERO
     if (query.moltype == MolType.PROTEIN and query.length <= RESIDUES_PER_THREAD) or \
        (query.moltype == MolType.NUCLEOTIDE and query.length <= BASES_PER_THREAD):
         return MTMode.ZERO
@@ -156,7 +125,7 @@ def get_mt_mode(program: str, options: str, db: DbData, query: SeqData) -> MTMod
     return MTMode.ZERO
 
 
-def get_num_cpus(program: str, mt_mode: MTMode, query: SeqData) -> int:
+def get_num_cpus(cloud_provider: CSP, program: str, mt_mode: MTMode, query: SeqData) -> int:
     """Get number of CPUs to use to optimally run BLAST
 
     Arguments:
@@ -164,13 +133,16 @@ def get_num_cpus(program: str, mt_mode: MTMode, query: SeqData) -> int:
         mt_mode: BLAST MT mode
         query: Queries information"""
     if mt_mode == MTMode.ZERO:
-        return NUM_THREADS_MT_ZERO
+        return NUM_THREADS_MT_ZERO_AWS if cloud_provider == CSP.AWS else NUM_THREADS_MT_ZERO_GCP
     else:
-        if program.lower() == 'blastx':
-            characters_per_thread = BASES_PER_THREAD_BLASTX
+        characters_per_thread = RESIDUES_PER_THREAD if query.moltype == MolType.PROTEIN else BASES_PER_THREAD
+        num_cpus = query.length // characters_per_thread
+        if query.length % characters_per_thread > 0:
+            num_cpus += 1
+        if cloud_provider == CSP.AWS:
+            return min([num_cpus, MAX_NUM_THREADS_AWS])
         else:
-            characters_per_thread = RESIDUES_PER_THREAD if query.moltype == MolType.PROTEIN else BASES_PER_THREAD
-        return min([query.length // characters_per_thread + 1, 16])
+            return min([num_cpus, MAX_NUM_THREADS_GCP])
 
 
 def get_batch_length(program: str, mt_mode: MTMode, num_cpus: int) -> int:
@@ -187,53 +159,98 @@ def get_batch_length(program: str, mt_mode: MTMode, num_cpus: int) -> int:
         raise UserReportError(returncode=INPUT_ERROR,
                               message=f"Invalid BLAST program '{program}'")
 
-    if program.lower == 'blastx':
-        batch_len = BASES_PER_THREAD_BLASTX
-
     if mt_mode == MTMode.ONE:
-        batch_len *= num_cpus
+        moltype = ElbSupportedPrograms().get_query_mol_type(program)
+        if moltype == MolType.UNKNOWN:
+            raise UserReportError(returncode=INPUT_ERROR,
+                                  message=f"Invalid BLAST program '{program}'")
+        characters_per_thread = BASES_PER_THREAD if moltype == MolType.NUCLEOTIDE else RESIDUES_PER_THREAD
+        batch_len = characters_per_thread * num_cpus
 
     return batch_len
 
 
-def aws_get_mem_limit(num_cpus: PositiveInteger, machine_type: str,
-                      db: DbData, db_factor: float) -> MemoryStr:
+def aws_get_mem_limit(num_cpus: PositiveInteger, 
+        machine_type: str,
+        db: Optional[DbData] = None, 
+        db_factor: float = 0.0, 
+        region: str = ELB_DFLT_AWS_REGION) -> float:
     """Get memory limit for searching a single query batch for AWS
 
     Arguments:
         num_cpus: Number of CPUs per search job
         db: Database information
-        machine_type: Machine type (can be "optimali")
-        db_factor: If larger than 0, memory limit will be computed as databse
+        machine_type: Machine type (can be "optimal")
+        db_factor: If larger than 0, memory limit will be computed as database
                    bytes-to-cache times db_factor
 
     Returns:
-        A search job memory limit as MemoryStr"""
+        A search job memory limit in GB as float"""
     if db_factor > 0.0:
-
-        return MemoryStr(f'{round(db.bytes_to_cache_gb * db_factor, 1)}G')
+        if not db:
+            raise ValueError('The "db" parameter must be provided if db_factor > 0.0')
+        return round(db.bytes_to_cache_gb * db_factor, 1)
     elif machine_type == 'optimal':
-        result = 60 if db.bytes_to_cache_gb >= 60 else db.bytes_to_cache_gb + 2
-        return MemoryStr(f'{result}G')
+        if not db:
+            raise ValueError('The "db" parameter must be provided if machine_type == "optimal"')
+        result = 60 if db.bytes_to_cache_gb >= 60 else db.bytes_to_cache_gb + MEMORY_FOR_BLAST_HITS
+        return result
     else:
-        props = aws_get_machine_properties(machine_type)
+        boto_cfg = create_aws_config(region)
+        props = aws_get_machine_properties(machine_type, boto_cfg)
         jobs_per_instance = math.floor(props.ncpus / num_cpus)
-        mem_limit = round((props.memory - SYSTEM_MEMORY_RESERVE) / jobs_per_instance, 1)
-        return MemoryStr(f'{mem_limit}G')
+        mem_limit = (props.memory - SYSTEM_MEMORY_RESERVE) / jobs_per_instance
+        return int(mem_limit * 10) / 10
 
 
-def gcp_get_mem_limit(db: DbData, db_factor: float) -> MemoryStr:
-    """Get memory limit for searching a single query batch for GCP. One job
-    per instance is assumed.
+def gcp_get_mem_limit(machine_type: str) -> float:
+    """Get memory limit for searching a single query batch for GCP. Kubernetes
+    schedules jobs based on CPU and memory request, so memory limit for each
+    job can be as high as instance RAM.
 
     Arguments:
+        machine_type: Machine type
+
+    Returns:
+        A search job memory limit int GB as float"""
+    try:
+        props = gcp_get_machine_properties(machine_type)
+    except NotImplementedError as err:
+        raise UserReportError(returncode=INPUT_ERROR,
+                              message=f'Invalid machine type. Machine type name "{machine_type}" is incorrect or not supported by ElasticBLAST: {str(err)}')
+
+    mem_limit = props.memory - SYSTEM_MEMORY_RESERVE
+    return mem_limit
+
+
+def get_mem_limit(cloud_provider: CSP, machine_type: str, num_cpus: PositiveInteger,
+        db: Optional[DbData] = None, db_factor: float = 0.0, cloud_region: str = ELB_DFLT_AWS_REGION) -> MemoryStr:
+    """Get memory limit for searching a single query batch, wrapper over
+    functions specialized for each cloud provider. See aws_get_mem_limit and
+    gcp_get_mem_limit for details on how memory limit is computed on each
+    platform.
+
+    Arguments:
+        cloud_provider: Cloud provider
+        machine_type: Instance type
+        num_cpus: Number of CPUs for a search job
         db: Database information
         db_factor: If larger than 0, memory limit will be computed as database
                    bytes-to-cache times db_factor
 
     Returns:
         A search job memory limit as MemoryStr"""
-    return MemoryStr(f'{round(db.bytes_to_cache_gb * db_factor, 1)}G')
+    region = cloud_region
+    if cloud_provider == CSP.AWS:
+        mem_limit = aws_get_mem_limit(num_cpus, machine_type, db, db_factor, cloud_region)
+    else:
+        mem_limit = gcp_get_mem_limit(machine_type)
+
+    if mem_limit <= 0:
+        raise UserReportError(returncode=INPUT_ERROR,
+                              message=f'The selected machine type {machine_type}: does not have enough memory to run the search. Please, select machine type with more memory.')
+
+    return MemoryStr(f'{mem_limit}G')
 
 
 def aws_get_machine_type(db: DbData, num_cpus: PositiveInteger, region: str) -> str:
